//
// NAME:
//      main.cpp
//PURPOSE:
//      Cluster detector - detects data points created by animal staying in approximately same location for a set period of time
//CALLING SEQUENCE:
//      run Cluster Detector.exe
//INPUTS:
//      csv file formated as: key field, animal ID, date identifier as an integer (e.g., julian date within year), x coordinate, y coordinate
//OUTPUTS:
//      csv file formated as: key field, animal ID, date identifier as an integer, calculated distance, animal ID, date identifier as an integer, key field
//
//  Created by Aleksandra Andic on 13 July 2011.
//  May be freely used and modified with attribution
//  <insert paper citation>
//
//MODIFICATION HISTORY:
//      06 Sep 2013 A.Andic - code rewriten to fit in one file and adjusted to be easily run in Windows OS, as demanded by journal
#include <iostream>
#include <map>
#include <fstream>
#include <strstream>
#include <cstdlib>
#include <cstring>
#include <string>
#include <cstdio>
#include <cmath>

using namespace std;

int main()
{
    cout <<" \n";
    cout <<"CLUSTER DETECTOR";
    cout <<" \n";
    cout <<" \n";
    cout <<"This code selects clusters of locations for each animal ID within\n";
    cout <<"a specified time interval and distance using an input file of \n";
    cout <<"Cartesian coordinates (e.g., UTMs).  In order to run correctly, \n";
    cout <<"ensure that input data are entered as comma-separated values as follows: \n";
    cout <<"key field, animal ID, date identifier as an integer (e.g., Julian date\n";
	cout <<"within year, i.e., '2014365' for 31 December 2014), x coordinate, y coordinate."<< endl;
    cout <<"Separate input values with coma and save as an ordinary tex file\n";
    cout <<"(e.g., *.csv or *.txt).\n";
    cout <<" \n";
    cout <<" \n";;

    //reading in the data and separating main variables


    map<int,string>m_keyfild;
    map<int,string>m_animalID;
    map<int,int>m_datum;
    map<int,float>m_xcoor;
    map<int,float>m_ycoor;
    map<int,bool>m_processed;

    cout<<"Please place input datafile in the same directory as this executable \n";
    cout<<"and TYPE in the name of the input datafile at the blinking cursor  \n";
    cout<<"below (e.g., 'myfile.txt' or 'LionGPS.csv' -Note: case-sensitive!) \n";
    cout<<"and enter: \n";
    cout <<" \n";
    cout <<" \n";
    string s_input;
    cout <<" \n";
    //s_input="LionExport.txt"; //"M:\\Distance selector\\LionExport.txt";
    cin >> s_input;

    int c,maincount, pomoc;
    float pom;
    char buffer1[2048];
    istrstream ostr1(buffer1, 2048);
    maincount=0;

    ifstream file (s_input.c_str());
    if (file.is_open())
       {
        while ( file.good() )
              {
              file.getline (buffer1,2048); //line);
              c=0;
              char * pch;
              pch = strtok (buffer1,",");

              while (pch != NULL)
                    {
                    if (c==0){m_keyfild[maincount]=pch;}
                    if (c==1){m_animalID[maincount]=pch;}
                    if (c ==2) {
                        pomoc = atoi(pch);
                        m_datum[maincount]=pomoc; }
                    if  (c ==3)
                        {pom = atof(pch);
                        m_xcoor[maincount] = pom;}
                    if  (c ==4)
                        {pom = atof(pch);
                        m_ycoor[maincount] = pom;}
                    pch = strtok (NULL, ",");
                    c=c+1;
                    }
                m_processed[maincount] = false;
                maincount=maincount+1;
                }
            file.close();
            }

    else cout << "Unable to open file";

    // "Setting the reference data ";
    int i_max_clusterdistance, i_max_unit,i_max_record, x;
    cout <<"Input the maximum radius defining the cluster as an integer value \n";
    cout<<"(suggested:200 spatial units)";
    cout <<" \n";
    cout <<" \n";
    cin >> i_max_clusterdistance;
    cout <<" \n";
    cout <<"Input the maximum number of temporal intervals (e.g., days) for the cluster";
    cout <<"(suggested: 6 units) as an integer value:";
    cout <<" \n";
    cout <<" \n";
    cin >> i_max_unit;
    cout <<" \n";

    // counting the different animals

     //map cannot be transfered as a variable to int subrutine?
    int b=0,d=0,e=0;
    c=0;
    map<int,string>m_differentanimals;
    m_differentanimals[0]=m_animalID[0];

    cout<<"Querying animals in dataset:"<<endl;
    cout <<" \n";
    while (b<maincount-1){
          //cout<<b<<m_animalID[b]<<endl;
          if(m_animalID[b] != m_animalID[b+1]){
                c=0;
                d=0;
                while(c <b){
                     if(m_animalID[b+1]==m_differentanimals[c]){d=d+1;}
                     c=c+1.;
                }
                if (d==0){
                    m_differentanimals[e]=m_animalID[b+1];
                    cout<<m_differentanimals[e]<<", ";
                    e=e+1;
                }
            }
            b=b+1;
        }
        int animalcount;
        animalcount=e;

        cout<<" "<<endl;

    //separating animals by their ID and comparing the data in those separations.
    cout <<" \n";
    cout<<"Starting cluster determination"<<endl;
    cout <<" \n";
    cout<<"Input name for the results file (results will be saved in the same directory"<<endl;
    cout<<"as  executable and input files):"<<endl;
    cout <<" \n";
    string s_out,s_outfile;
    cin >>s_out;
    //s_out="clusters2.txt";
    s_outfile=s_out;


    int a=0,datacount=0,br=0,i_date,i_date_kr;
    string animalid;
    double pomo,d_ref_distance, d_calc_distance, d_dif_distance;

    ofstream myfile (s_outfile.c_str());
        if (myfile.is_open())
        {

    br=0;
    while(br<animalcount){
    animalid=m_differentanimals[br];
    map<int,string>m_keyfild_sep;
    map<int,int>m_datum_sep;
    map<int,float>m_xcoor_sep;
    map<int,float>m_ycoor_sep;
    map<int,double>m_distance;
    map<int,bool>m_processed_sep;

    cout<<"Processing animal:"<<animalid<<endl;
    b=0;
    a=0;
    while(a<maincount){
            if (animalid==m_animalID[a]){
                m_datum_sep[b]=m_datum[a];
                m_keyfild_sep[b]=m_keyfild[a];
                m_xcoor_sep[b]=m_xcoor[a];
                m_ycoor_sep[b]=m_ycoor[a];
                pomo=m_xcoor[a]*m_xcoor[a]+m_ycoor[a]*m_ycoor[a];
                m_distance[b]=sqrt(pomo);
                b=b+1;
            }
            a=a+1;
    }
    datacount=b;
    int f;
    c=0;
    while(c<datacount){
      i_date=m_datum_sep[c];
      i_date_kr=i_date+i_max_unit;
      d_ref_distance=m_distance[c];
      f=c;
      while(f<datacount){
            if(m_processed_sep[f]==false){
              if(m_datum_sep[f]>i_date){
              if(m_datum_sep[f]<i_date_kr){
                d_calc_distance=m_distance[f];
                d_dif_distance=d_ref_distance-d_calc_distance;
                if (d_dif_distance<0){d_dif_distance=d_dif_distance*(-1);}
                if (d_dif_distance<i_max_clusterdistance){
                    cout<<"Cluster condition satisfied in time interval: "<<i_date<<" - "<<i_date_kr<<"for animal: "<<animalid<<endl;
                    myfile <<m_keyfild_sep[c]<<","<<animalid<<","<<m_datum_sep[c]<<","<<d_dif_distance<<","<<m_datum_sep[f]<<","<<m_keyfild_sep[f]<<"\n";
                    m_processed_sep[f]=true;
                }
               }
               }

            }
       f=f+1;
       }
    c=c+1;
    }
    br=br+1;
    }
    }
    cout <<"\n";
    cout <<"\n";
    cout <<"Cluster analysis complete. Results saved to file: "<<s_out<<"\n";
    return 0;
}
